import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.LinkedList;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * Leinwand ist eine Klasse, die einfache Zeichenoperationen auf einer
 * leinwandartigen Zeichenflche ermglicht. Sie ist eine vereinfachte Version
 * der Klasse Canvas (englisch fr Leinwand) des JDK und wurde speziell fr das
 * Projekt "Figuren" geschrieben.
 * 
 * 
 * @author: Bruce Quig
 * @author: Michael Klling (mik)
 * @author: Axel Schmolitzky
 * 
 * 
 * In der erweiterten Fassung kann die Leinwand zustzlich auch Texte darstellen
 * und Tastaturbefehle abfragen. Dadurch ist es mglich, einfache Spiele auf der
 * Leinwand zu programmieren. Die Farbpalette der Methode setzeZeichenfarbe ()
 * wird schrittweise erweitert, um den Anwendungsbedrfnissen gerecht zu werden.
 * 
 * 
 * @author: Bernd Gramlich
 * 
 * @version: 2012-10-06
 */
public class Leinwand {
	// Hinweis: Die Implementierung dieser Klasse (insbesondere die
	// Verwaltung der Farben und Identitten der Figuren) ist etwas
	// komplizierter als notwendig. Dies ist absichtlich so, weil damit
	// die Schnittstellen und Exemplarvariablen der Figuren-Klassen
	// fr den Lernanspruch dieses Projekts einfacher und klarer
	// sein knnen.

	private static Leinwand leinwandSingleton;

	/**
	 * Fabrikmethode, die eine Referenz auf das einzige Exemplar dieser Klasse
	 * zurckliefert. Wenn es von einer Klasse nur genau ein Exemplar gibt, wird
	 * dieses als 'Singleton' bezeichnet.
	 */
	public static Leinwand gibLeinwand() {
		if (leinwandSingleton == null) {
			leinwandSingleton = new Leinwand("Wrfelspiel 10b",
			        1500, 1500, Color.white);
		}
		leinwandSingleton.setzeSichtbarkeit(true);
		return leinwandSingleton;
	}

	// ----- Exemplarvariablen -----

	private JFrame fenster;

	private Zeichenflaeche zeichenflaeche;

	private Graphics2D graphic;

	private Color hintergrundfarbe;

	private Image leinwandImage;

	private List<Object> figuren;
	
	private List<Object> texte;

	private Map<Object, ShapeMitFarbe> figurZuShape;

	private Map<Object, TextMitFarbe> textZuString;
	
	private LinkedList<KeyEvent> tastenbefehle;
	
	/**
	 * Erzeuge eine Leinwand.
	 * 
	 * @param titel
	 *            Titel, der im Rahmen der Leinwand angezeigt wird
	 * @param breite
	 *            die gewnschte Breite der Leinwand
	 * @param hoehe
	 *            die gewnschte Hhe der Leinwand
	 * @param grundfarbe
	 *            die Hintergrundfarbe der Leinwand
	 */
	private Leinwand(String titel, int breite, int hoehe, Color grundfarbe) {
		fenster = new JFrame();
		zeichenflaeche = new Zeichenflaeche();
		fenster.setContentPane(zeichenflaeche);
		fenster.setTitle(titel);
		fenster.addKeyListener(new Horcher ());
		zeichenflaeche.setPreferredSize(new Dimension(breite, hoehe));
		hintergrundfarbe = grundfarbe;
		fenster.pack();
		figuren = new ArrayList<Object>();
		figurZuShape = new HashMap<Object, ShapeMitFarbe>();
		texte = new ArrayList<Object>();
		textZuString = new HashMap<Object, TextMitFarbe>();
		tastenbefehle = new LinkedList<KeyEvent>();
	}

	/**
	 * Setze, ob diese Leinwand sichtbar sein soll oder nicht. Wenn die Leinwand
	 * sichtbar gemacht wird, wird ihr Fenster in den Vordergrund geholt. Diese
	 * Operation kann auch benutzt werden, um ein bereits sichtbares
	 * Leinwandfenster in den Vordergrund (vor andere Fenster) zu holen.
	 * 
	 * @param sichtbar
	 *            boolean fr die gewnschte Sichtbarkeit: true fr sichtbar,
	 *            false fr nicht sichtbar.
	 */
	public void setzeSichtbarkeit(boolean sichtbar) {
		if (graphic == null) {
			// erstmaliger Aufruf: erzeuge das Bildschirm-Image und flle
			// es mit der Hintergrundfarbe
			Dimension size = zeichenflaeche.getSize();
			leinwandImage = zeichenflaeche.createImage(size.width, size.height);
			graphic = (Graphics2D) leinwandImage.getGraphics();
			graphic.setColor(hintergrundfarbe);
			graphic.fillRect(0, 0, size.width, size.height);
			graphic.setColor(Color.black);
		}
		fenster.setVisible(sichtbar);
	}

	/**
	 * Zeichne fr das gegebene Figur-Objekt eine Java-Figur (einen Shape) auf
	 * die Leinwand.
	 * 
	 * @param figur
	 *            das Figur-Objekt, fr das ein Shape gezeichnet werden soll
	 * @param farbe
	 *            die Farbe der Figur
	 * @param shape
	 *            ein Objekt der Klasse Shape, das tatschlich gezeichnet wird
	 */
	public void zeichne(Object figur, String farbe, Shape shape) {
		figuren.remove(figur); // entfernen, falls schon eingetragen
		figuren.add(figur); // am Ende hinzufgen
		figurZuShape.put(figur, new ShapeMitFarbe(shape, farbe));
		erneutZeichnen();
	}

	/**
	 * Entferne die gegebene Figur von der Leinwand.
	 * 
	 * @param figur
	 *            die Figur, deren Shape entfernt werden soll
	 */
	public void entferne(Object figur) {
		figuren.remove(figur); // entfernen,falls schon eingetragen
		figurZuShape.remove(figur);
		erneutZeichnen();
	}

	/**
	 * Schreibe fr das gegebene Text-Objekt eine Zeichenkette (einen String) auf
	 * die Leinwand.
	 * 
	 * @param text
	 *            das Text-Objekt, fr das ein String geschrieben werden soll
	 * @param farbe
	 *            die Farbe des Textes
	 * @param zeichenkette
	 *            ein Objekt der Klasse String, das tatschlich geschrieben wird
	 * @param x
	 *            x-Koordinate des Textes
	 * @param y
	 *            y-Koordinate des Textes
	 * @param groesse
	 *            Schriftgroesse des Textes
	 */
	public void schreibe(Object text, String farbe, String zeichenkette, int x, int y, int groesse) {
		texte.remove(text); // entfernen, falls schon eingetragen
		texte.add(text); // am Ende hinzufgen
		textZuString.put(text, new TextMitFarbe(zeichenkette, farbe, x, y, groesse));
		erneutZeichnen();
	}

	/**
	 * Radiere den gegebenen Text von der Leinwand.
	 * 
	 * @param text
	 *            der Text, dessen Zeichenkette entfernt werden soll
	 */
	public void radiere(Object text) {
		texte.remove(text); // entfernen,falls schon eingetragen
		textZuString.remove(text);
		erneutZeichnen();
	}
	/**
	 * Setze die Zeichenfarbe der Leinwand.
	 * 
	 * @param farbname
	 *            der Name der neuen Zeichenfarbe.
	 */
	public void setzeZeichenfarbe(String farbname) {
		if (farbname.equals("rot")) {
			graphic.setColor(Color.red);
		} else if (farbname.equals("schwarz")) {
			graphic.setColor(Color.black);
		} else if (farbname.equals("blau")) {
			graphic.setColor(Color.blue);
		} else if (farbname.equals("gelb")) {
			graphic.setColor(Color.yellow);
		} else if (farbname.equals("grn")) {
			graphic.setColor(Color.green);
		} else if (farbname.equals("lila")) {
			graphic.setColor(Color.magenta);
		} else if (farbname.equals("wei")) {
			graphic.setColor(Color.white);
		} else if (farbname.equals("grau")) {
			graphic.setColor(Color.gray);
		} else if (farbname.equals("orange")) {
			graphic.setColor(Color.orange);
		} else if (farbname.equals("trkis")) {
			graphic.setColor(Color.cyan);
		} else if (farbname.equals("pink")) {
			graphic.setColor(Color.pink);
		} else if (farbname.equals("braun")) {
			graphic.setColor(new Color (102, 51, 0));
		} else if (farbname.equals("bleich")) {
			graphic.setColor(new Color (248, 248, 190));
		} else if (farbname.equals("dunkelbraun")) {
			graphic.setColor(new Color (76, 25, 0));
		} else if (farbname.equals("hellbraun")) {
			graphic.setColor(new Color (153, 102, 51));
		} else if (farbname.equals("blond")) {
			graphic.setColor(new Color (255, 211, 51));
		} else if (farbname.equals("gebrunt")) {
			graphic.setColor(new Color (255, 222, 194));
		} else {
			graphic.setColor(Color.black);
		}
	}

	/**
	 * Warte fr die angegebenen Millisekunden. Mit dieser Operation wird eine
	 * Verzgerung definiert, die fr animierte Zeichnungen benutzt werden kann.
	 * 
	 * @param millisekunden
	 *            die zu wartenden Millisekunden
	 */
	public void warte(int millisekunden) {
		try {
			Thread.sleep(millisekunden);
		} catch (Exception e) {
			// Exception ignorieren
		}
	}

	/**
	 * Zeichne erneut alle Figuren auf der Leinwand.
	 */
	private void erneutZeichnen() {
		loeschen();
		for (Object figur : figuren) {
			figurZuShape.get(figur).draw(graphic);
		}
		for (Object text : texte) {
			textZuString.get(text).write(graphic);
		}
		zeichenflaeche.repaint();
	}

	/**
	 * Lsche die gesamte Leinwand.
	 */
	private void loeschen() {
		Color original = graphic.getColor();
		graphic.setColor(hintergrundfarbe);
		Dimension size = zeichenflaeche.getSize();
		graphic.fill(new Rectangle(0, 0, size.width, size.height));
		graphic.setColor(original);
	}
	
	/**
	 * Gib den letzten, noch nicht verarbeiteten Tastendruck zurck,
	 * den die Leinwand empfangen hat. Falls kein Tastendruck vorkam,
	 * wird null zurckgegeben.
	 */
	public KeyEvent tastendruck() {
	    warte(5);
	    if (tastenbefehle.size () == 0) {
	        return null;
	    }
	    else {
	        KeyEvent tastenbefehl = tastenbefehle.getFirst();
	        tastenbefehle.removeFirst();
	        return tastenbefehl;
	    }
	}
	        
	/***************************************************************************
	 * Interne Klasse Zeichenflaeche - die Klasse fr die GUI-Komponente, die
	 * tatschlich im Leinwand-Fenster angezeigt wird. Diese Klasse definiert
	 * ein JPanel mit der zustzlichen Mglichkeit, das auf ihm gezeichnet Image
	 * aufzufrischen (erneut zu zeichnen).
	 */
	private class Zeichenflaeche extends JPanel {
		private static final long serialVersionUID = 20060330L;

		public void paint(Graphics g) {
			g.drawImage(leinwandImage, 0, 0, null);
		}
	}

	/***************************************************************************
	 * Interne Klasse ShapeMitFarbe - Da die Klasse Shape des JDK nicht auch
	 * eine Farbe mitverwalten kann, muss mit dieser Klasse die Verknpfung
	 * modelliert werden.
	 */
	private class ShapeMitFarbe {
		private Shape shape;

		private String farbe;

		public ShapeMitFarbe(Shape shape, String farbe) {
			this.shape = shape;
			this.farbe = farbe;
		}

		public void draw(Graphics2D graphic) {
			setzeZeichenfarbe(farbe);
			graphic.fill(shape);
		}
	}

	/***************************************************************************
	 * Interne Klasse TextMitFarbe - Da die Klasse String des JDK nicht auch
	 * eine Farbe mitverwalten kann, muss mit dieser Klasse die Verknpfung
	 * modelliert werden.
	 */
	private class TextMitFarbe {
		private String zeichenkette;

		private String farbe;
		
		private int x, y;
		private int groesse;

		public TextMitFarbe(String zeichenkette, String farbe, int x, int y, int groesse) {
			this.zeichenkette = zeichenkette;
			this.farbe = farbe;
			this.x = x;
			this.y = y;
			this.groesse = groesse;
		}

		public void write(Graphics2D graphic) {
		    Font f = new Font("Serif", 0, groesse);
		    
			setzeZeichenfarbe(farbe);
			graphic.setFont(f);
			graphic.drawString(zeichenkette, x, y);
		}
	}
	
	/****************************************************************************
	 * Interne Klasse Horcher - Speichert alle Tastaturereignisse in der Schlange
	 * tastenbefehle
	 */
	private class Horcher extends KeyAdapter {
        public void keyPressed (KeyEvent taste) {
           tastenbefehle.addLast (taste);
        }
    }
}
