import java.awt.event.KeyEvent; 

/**
 * Kniffelspiel fr einen Spieler.
 * 
 * @author Bernd Gramlich 
 * @version 2013-02-01
 */
public class Spiel
{
    // Ein Feld von fnf Wrfeln.
    private Wuerfel[] spielwuerfel;
    
    // Wrfelergebnis als Anzahl der 1er, 2er, ..., 6er
    private int[] anzahl;
    
    // Die Namen der verschiedenen Wurfergebnisse.
    private String[] wurfname;
    private Beschriftung[] wurfAnzeige;
    
    // Die Tastaturkrzel der verschiedenen Wurfergebnisse.
    private char[] kuerzel;
    private Beschriftung[] kuerzelAnzeige;
    
    // Die Punktzahl fr die einzelnen Eintragungen.
    private int[] punktzahl;
    private Beschriftung[] punktAnzeige;
    
    // Ist ein Wurfergebnis schon verbraucht?
    // (D. h., ist dort schon etwas eingetragen?)
    private boolean[] verbraucht;
    
    // Interne Namen der Wurfergebnisse.
    private static final int EINSER = 0;
    private static final int ZWEIER = 1;
    private static final int DREIER = 2;
    private static final int VIERER = 3;
    private static final int FUENFER = 4;
    private static final int SECHSER = 5;
    private static final int BONUS = 6;
    private static final int DRILLING = 7;
    private static final int VIERLING = 8;
    private static final int FULLHOUSE = 9;
    private static final int KLEINESTRASSE = 10;
    private static final int GROSSESTRASSE = 11;
    private static final int KNIFFEL = 12;
    private static final int CHANCE = 13;
    private static final int ERGEBNIS = 14;
    private static final int WUERFELN = 15;
    
    // Buchstabenkrzel des ersten Wrfels.
    private static final char KLEINSTERWUERFEL = 'm';
    
    // Rundenzhler zur Orientierung fr den Spieler.
    private Beschriftung rundenzaehler;
    
    /**
     * Konstruktor.
     * Erzeuge die fnf Wrfel, platziere sie nebeneinander
     * und nummeriere sie mit 'm' bis 'q'.
     */
    public Spiel()
    {
        // Die Spielwrfel sind nummeriert von 0 bis 4.
        spielwuerfel = new Wuerfel[5];
        
        // Die Wrfelanzahlen sind nummeriert von 1 bis 6.
        anzahl = new int[7];
        
        int i;
        
        for (i = 0; i <= 4; i++)
        {
            // Erzeuge Spielwrfel mit der Nummer i.
            spielwuerfel[i] = new Wuerfel (i*120 + 10, 60, "lila", "schwarz", (char) (KLEINSTERWUERFEL + i));
        }
        
        werteAus ();
        
        // Erzeuge alle Felder fr die Ergebnisliste.
        wurfname          = new String[16];
        wurfAnzeige       = new Beschriftung[16];
        kuerzel           = new char[16];
        kuerzelAnzeige    = new Beschriftung[16];
        punktzahl         = new int[16];
        punktAnzeige      = new Beschriftung[16];
        verbraucht        = new boolean[16];
              
        int n;
        
        // Lege die Namen und Krzel der einzelnen Ergebnisse fest.
        wurfname[EINSER]        = "1er";           kuerzel[EINSER]        = '1';
        wurfname[ZWEIER]        = "2er";           kuerzel[ZWEIER]        = '2';
        wurfname[DREIER]        = "3er";           kuerzel[DREIER]        = '3';
        wurfname[VIERER]        = "4er";           kuerzel[VIERER]        = '4';
        wurfname[FUENFER]       = "5er";           kuerzel[FUENFER]       = '5';
        wurfname[SECHSER]       = "6er";           kuerzel[SECHSER]       = '6';
        wurfname[BONUS]         = "Bonus";         kuerzel[BONUS]         = 'b';
        wurfname[DRILLING]      = "Drilling";      kuerzel[DRILLING]      = 'd';
        wurfname[VIERLING]      = "Vierling";      kuerzel[VIERLING]      = 'v';
        wurfname[FULLHOUSE]     = "Full House";    kuerzel[FULLHOUSE]     = 'f';
        wurfname[KLEINESTRASSE] = "Kleine Strae"; kuerzel[KLEINESTRASSE] = 's';
        wurfname[GROSSESTRASSE] = "Groe Strae";  kuerzel[GROSSESTRASSE] = 'g';
        wurfname[KNIFFEL]       = "Kniffel";       kuerzel[KNIFFEL]       = 'k';
        wurfname[CHANCE]        = "Chance";        kuerzel[CHANCE]        = 'c';
        wurfname[ERGEBNIS]      = "Ergebnis";      kuerzel[ERGEBNIS]      = 'e';
        wurfname[WUERFELN]      = "Wrfeln";       kuerzel[WUERFELN]      = 'w';
                
        // Erzeuge die Beschriftungsfelder.       
        for (n = 0; n <= 15; n++)
        {
            wurfAnzeige[n] = new Beschriftung ();
            wurfAnzeige[n].setzeInhalt (wurfname[n]);
            wurfAnzeige[n].setzeSchriftgroesse (18);
            wurfAnzeige[n].setzeFarbe ("schwarz");
            wurfAnzeige[n].setzePosition (25, 200 + 20*n);
            
            kuerzelAnzeige[n] = new Beschriftung ();
            kuerzelAnzeige[n].setzeInhalt (kuerzel[n] + ")");
            kuerzelAnzeige[n].setzeSchriftgroesse (18);
            kuerzelAnzeige[n].setzeFarbe ("schwarz");
            kuerzelAnzeige[n].setzePosition (5, 200 + 20*n);
            
            punktAnzeige[n] = new Beschriftung ();
            punktAnzeige[n].setzeSchriftgroesse (18);
            punktAnzeige[n].setzeFarbe ("schwarz");
            punktAnzeige[n].setzePosition (150, 200 + 20*n);
        }
                        
        // Erzeuge den Rundenzhler.
        rundenzaehler = new Beschriftung ();
        rundenzaehler.setzeInhalt ("Kniffel");
        rundenzaehler.setzeSchriftgroesse (24);
        rundenzaehler.setzeFarbe ("grau");
        rundenzaehler.setzePosition (1, 40);
        rundenzaehler.macheSichtbar ();
    }
    
    /**
     * Wrfle mit allen fnf Spielwrfeln.
     */
    public void wuerfle ()
    {
        for (int i = 0; i <= 4; i++)
        {
            /*
             * Wrfle den Spielwrfel mit der Nummer i.
             */
            spielwuerfel[i].wuerfle ();
        }
        
        werteAus ();
    }
    
    /**
     * Gib fr alle Wrfel die Statistik aus.
     */
    public void druckeStatistik ()
    {
        int i;
        
        for (i = 0; i <= 4; i++)
        {
            /*
             * Gib die Statistik des Spielwrfels mit der Nummer i aus.
             */
            System.out.println ();
            System.out.println ("Wrfel " + i + ":");
            spielwuerfel[i].druckeStatistik ();
        }
    }
    
    /**
     * Werte den Wurf aus, d. h. zhle, wie oft jede Augenzahl geworfen wurde.
     * 
     * anzahl[1] ist die Anzahl der 1er,
     * anzahl[2] ist die Anzahl der 2er usw.
     */
    private void werteAus ()
    {  
        /*
         * Setze die Anzahl der 1er, 2er, 3er, ... jeweils auf 0.
         */       
        for (int j = 1; j <= 6; j++)
        {
            anzahl[j] = 0;
        }
        
        /*
         * Frage von jedem Wrfel (0 bis 4) jeweils die Augenzahl ab.
         * Erhhe die Anzahl des Vorkommens dieser Augenzahl um 1.
         */
        for (int i = 0; i <= 4; i++)
        {
            anzahl[spielwuerfel[i].gibAugenzahl ()] ++;
        }
    }
    
    /**
     * Gib an, ob ein Drilling vorkommt.
     */
    public boolean hatDrilling ()
    {
        boolean gefunden = false;
        
        for (int j = 1; j <= 6; j++)
        {
            if (anzahl[j] >= 3)
            {
                gefunden = true;
            }
        }
        
        return gefunden;
    }
    
    /**
     * Gib an, ob ein Vierling vorkommt.
     */
    public boolean hatVierling ()
    {
        boolean gefunden = false;
        
        for (int j = 1; j <= 6; j++)
        {
            if (anzahl[j] >= 4)
            {
                gefunden = true;
            }
        }
        
        return gefunden;
    }
    
    /**
     * Gib an, ob das Ergebnis ein Kniffel ist.
     */
    public boolean istKniffel ()
    {
        boolean gefunden = false;
        
        for (int j = 1; j <= 6; j++)
        {
            if (5 == anzahl[j])
            {
                gefunden = true;
            }
        }
        
        return gefunden;
    }
    
    /**
     * Gib an, ob ein Full House vorliegt.
     */
    public boolean istFullHouse ()
    {
        boolean gefunden = false;
        
        for (int j = 1; j <= 6; j++)
        {
            /*
             * Wenn ein Drilling vorliegt ...
             */
            if (3 == anzahl[j])
            {
                for (int k = 1; k <= 6; k++)
                {
                    /*
                     * ... und noch ein *anderer* Zwilling vorliegt ...
                     */
                    if (2 == anzahl[k] && k != j)
                    {
                        /*
                         * ... dann haben wir Full House.
                         */
                        gefunden = true;
                    }
                }
            }
        }
        
        return gefunden;
    }
    
    /**
     * Gib an, ob eine groe Strae vorliegt.
     */
    public boolean istGrosseStrasse ()
    {
        boolean gefunden = false;
        
        if (1 == anzahl[1] && 1 == anzahl[2] && 1 == anzahl[3]
                           && 1 == anzahl[4] && 1 == anzahl[5])
        {
            gefunden = true;
        }
        
        if (1 == anzahl[6] && 1 == anzahl[2] && 1 == anzahl[3]
                           && 1 == anzahl[4] && 1 == anzahl[5])
        {
            gefunden = true;
        }
        
        return gefunden;
    }
    
    
    /**
     * Gib an, ob eine kleine Strae enthalten ist.
     */
    public boolean hatKleineStrasse ()
    {
        boolean gefunden = false;
        
        if (1 <= anzahl[1] && 1 <= anzahl[2] && 1 <= anzahl[3] && 1 <= anzahl[4])
        {
            gefunden = true;
        }
        
        if (1 <= anzahl[2] && 1 <= anzahl[3] && 1 <= anzahl[4] && 1 <= anzahl[5])
        {
            gefunden = true;
        }
        
        if (1 <= anzahl[3] && 1 <= anzahl[4] && 1 <= anzahl[5] && 1 <= anzahl[6])
        {
            gefunden = true;
        }
        
        return gefunden;
    }
    
    /**
	 * Spiele.
	 * Mit Tastaturbefehlen kann man wrfeln, einzelne Wrfel markieren
	 * und Ergebnisse eintragen.
	 */
	public void spiele ()
	{
	    /*
	     * Hilfsvariablen:
	     * wuerfeltisch:  Die Leinwand, auf der wir spielen.
	     * runde:         Die Nummer der laufenden Spielrunde.
	     * wurf:          Die Nummer des Wurfs in der laufenden Spielrunde.
	     * i:             Wrfelnummer.
	     * n:             Ergebnisnummer.
	     * befehl:        Nummer des ausgewhlten Ergebnisses.
	     * taste:         Die letzte Taste, die auf der Leinwand gedrckt wurde
	     *                (als Tastaturereignis).
	     * buchstabe:     Der Buchstabe (oder das Zeichen) auf der gedrckten Taste.
	     * aufhoeren:     Will der Spieler aufhren?
	     */
	    Leinwand wuerfeltisch = Leinwand.gibLeinwand ();
	    int runde;
	    int wurf;
	    int i;
	    int n;       
        int befehl;
        KeyEvent taste;
        char buchstabe;
        boolean aufhoeren;       

        
        /*
         * Lsche alle Tasten, die vor Spielbeginn gedrckt wurden.
         */
        while ((taste = wuerfeltisch.tastendruck ()) != null);
        
        /*
         * Der Benutzer will mindestens einmal spielen.
         */
        aufhoeren = false;
        
        /*
         * Solange der Benutzer spielen will ...
         */
        while (!aufhoeren)
        {
            /*
             * Mache die Eintrge der Ergebnisliste sichtbar.
             */
        
            for (n = 0; n <= 15; n++)
            {            
                wurfAnzeige[n].macheSichtbar ();
            
                /*
                 * Bonus und Ergebnis knnen nicht direkt eingetragen werden.
                 */
                if ((n != BONUS) && (n != ERGEBNIS))
                {
                    verbraucht[n] = false;
                    kuerzelAnzeige[n].macheSichtbar ();
                }
                else
                {
                    verbraucht[n] = true;
                }
            
                /*
                 * Punktzahlen sind vor der Eintragung unsichtbar.
                 */
                punktzahl[n] = 0;
                punktAnzeige[n].setzeInhalt ("" + punktzahl[n]);
                punktAnzeige[n].macheUnsichtbar ();           
            }
        
            /*
             * Der Zwischenstand wird in schwarzer Farbe angezeigt.
             */
            punktAnzeige[ERGEBNIS].setzeFarbe("schwarz");
            punktAnzeige[ERGEBNIS].macheSichtbar ();
            
            /*
             * Spiele 13 Spielrunden fr die 13 Eintragungen.
             */
            for (runde = 1; runde <= 13; runde++)
            {            
                /*
                 * Markiere zu Beginn jeder Runde alle fnf Wrfel.
                 */
                for (i = 0; i <= 4; i++)
                {
                    spielwuerfel[i].markiere ();
                }
            
                /*
                 * Erlaube drei Wrfe je Spielrunde.
                 */
                for (wurf = 1; wurf <= 3; wurf ++)
                {
                    /*
                     * Zeige an, in welcher Spielrunde und bei welchem Wurf wir sind.
                     */
                    rundenzaehler.setzeInhalt ("Runde " + runde + ", Wurf " + wurf + ":");
                    
                    /*
                     * Wrfle alle markierten wrfel.
                     */
                    wuerfle ();
                
                    /*
                     * Erlaube erneutes Wrfeln nach dem ersten und zweiten, aber nicht
                     * nach dem dritten Wurf.
                     */
                    if (3 == wurf)
                    {
                        verbraucht[WUERFELN] = true;
                        wurfAnzeige[WUERFELN].macheUnsichtbar ();
                        kuerzelAnzeige[WUERFELN].macheUnsichtbar ();
                    }
                    else
                    {
                        verbraucht[WUERFELN] = false;
                        wurfAnzeige[WUERFELN].macheSichtbar ();
                        kuerzelAnzeige[WUERFELN].macheSichtbar ();
                    }
                
                    /*
                     * Beginne mit einem ungltigen Befehl.
                     */
                    befehl = -1;
                
                    /*
                     * Solange kein gltiger Befehl vorliegt ...
                     */
                    while (befehl < 0)
                    {
                        /*
                         * Warte, bis der Spieler eine Taste drckt.
                         */
                        while ((taste = wuerfeltisch.tastendruck ()) == null);
                        
                        /*
                         * Bestimme den Buchstaben, der auf der Taste steht.
                         */
                        buchstabe = taste.getKeyChar ();
                        
                        /*
                         * Wenn der Buchstabe zu einem Wrfel gehrt ...
                         */
                        i         = (int) (buchstabe - KLEINSTERWUERFEL);
                        
                        if (( i >= 0) && (i <= 4))
                        {
                            /*
                             * ... ndere die Markierung des Wrfels, 
                             */
                            spielwuerfel[i].aendereMarkierung ();
                        }
                        else
                        {
                            /*
                             * ... ansonsten finde heraus, ob der Buchstabe einem Befehl entspricht.
                             */
                            for (n = 0; n <= 15; n++)
                            {
                                if ((kuerzel [n] == buchstabe) && !verbraucht [n])
                                {
                                    befehl = n;
                                }
                            }
                        }
                    }
                    
                    /*
                     * Falls der Befehl einer Eintragung in die Ergebnisliste entspricht, vergebe
                     * die passende Punktzahl.
                     */
                    switch (befehl)
                    {
                        case EINSER: 
                        punktzahl[EINSER] = 1;
                        break;
                    
                        case ZWEIER:
                        punktzahl[ZWEIER] = 1;
                        break;
                    
                        case DREIER:
                        punktzahl[DREIER] = 1;
                        break;
                    
                        case VIERER:
                        punktzahl[VIERER] = 1;
                        break;
                    
                        case FUENFER:
                        punktzahl[FUENFER] = 1; 
                        break;
                    
                        case SECHSER:
                        punktzahl[SECHSER] = 1;
                        break;
                    
                        case DRILLING:
                        punktzahl[DRILLING] = 1;
                        break;
                    
                        case VIERLING:
                        punktzahl[VIERLING] = 1;
                        break;
                    
                        case FULLHOUSE:
                        punktzahl[FULLHOUSE] = 1;
                        break;
                    
                        case KLEINESTRASSE:
                        punktzahl[KLEINESTRASSE] = 1;
                        break;
                    
                        case GROSSESTRASSE:
                        punktzahl[GROSSESTRASSE] = 1;
                        break;
                    
                        case KNIFFEL:
                        punktzahl[KNIFFEL] = 1;
                        break;
                    
                        case CHANCE:
                        punktzahl[CHANCE] = 1;
                        break;
                    }
                
                    /*
                     * Falls der Befehl nicht "Wrfeln" lautet, ...
                     */
                    if (befehl != WUERFELN)
                    {
                        /*
                         * ... erlaube in dieser Spielrunde keinen weiteren Wurf.
                         */
                        wurf = 3;
                        
                        /*
                         * Markiere den Befehl als verbraucht.
                         */
                        verbraucht[befehl] = true;
                        kuerzelAnzeige[befehl].macheUnsichtbar ();
                    
                        /*
                         * Zeige die Punktzahl zu diesem Befehl an.
                         */
                        punktAnzeige[befehl].setzeInhalt ("" + punktzahl[befehl]);
                        punktAnzeige[befehl].macheSichtbar ();
                    
                        /*
                         * Aktualisiere den Bonus.
                         */
                        if (punktzahl[EINSER] + punktzahl[ZWEIER] + punktzahl[DREIER]
                            + punktzahl[VIERER] + punktzahl[FUENFER] + punktzahl[SECHSER] >= 63)
                        {
                            punktzahl[BONUS] = 35;
                            punktAnzeige[BONUS].setzeInhalt ("" + punktzahl[BONUS]);
                            punktAnzeige[BONUS].macheSichtbar ();
                        }
                    
                        /*
                         * Aktualisiere die Gesamtpunktzahl.
                         */
                        punktzahl[ERGEBNIS] = punktzahl[EINSER] + punktzahl[ZWEIER] + punktzahl[DREIER]
                                              + punktzahl[VIERER] + punktzahl[FUENFER] + punktzahl[SECHSER]
                                              + punktzahl[BONUS] + punktzahl[DRILLING] + punktzahl[VIERLING]
                                              + punktzahl[FULLHOUSE] + punktzahl[KLEINESTRASSE]
                                              + punktzahl[GROSSESTRASSE] + punktzahl[KNIFFEL]
                                              + punktzahl[CHANCE];                                         
                        punktAnzeige[ERGEBNIS].setzeInhalt ("" + punktzahl[ERGEBNIS]);
                    }           
                }
            }
        
            /*
             * Zeige nach Spielende das Wrfeln nicht mehr als Mglichkeit an.
             */
            verbraucht[WUERFELN] = true;
            wurfAnzeige[WUERFELN].macheUnsichtbar ();
            kuerzelAnzeige[WUERFELN].macheUnsichtbar ();
        
            /*
             * Der Endstand wird rot angezeigt.
             */
            punktAnzeige[ERGEBNIS].setzeFarbe("rot");
        
            /*
             * Informiere den Spieler, wie er weiterspielen kann.
             */
            rundenzaehler.setzeInhalt ("-- Ende mit <Eingabetaste>, neues Spiel mit anderer Taste. --");
            
            /*
             * Warte auf einen Tastendruck.
             */
            while ((taste = wuerfeltisch.tastendruck ()) == null);
                        
            /*
             * Spiele weiter auer bei der Eingabetaste.
             */            
            if (taste.getKeyCode () == KeyEvent.VK_ENTER)
            {
                aufhoeren = true;
            }
        }
        
        /*
         * Mache die Eintrge der Ergebnisliste unsichtbar.
         */        
        for (n = 0; n <= 15; n++)
        {            
            wurfAnzeige[n].macheUnsichtbar ();
            kuerzelAnzeige[n].macheUnsichtbar ();
            punktAnzeige[n].macheUnsichtbar ();           
        }
        
        rundenzaehler.setzeInhalt ("Kniffel");
	}
}