/**
 * Ein einzelner Spielwrfel.
 * 
 * @author Bernd Gramlich 
 * @version 2013-02-04
 */
public class Wuerfel
{
    /*
     * Die Augenzahl des Wrfels.
     */
    private int augenzahl;

    /*
     * Markierte Wrfel werden nicht geworfen.
     */
    private boolean markiert;

    /* 
     * Feld, das die Hufigkeit der Augenzahlen mitzhlt. 
     */
    private int[] haeufigkeit;
    /*
     * Referenzattribute zum Zeichnen.
     */
    private Rechteck wuerfelflaeche;
    private Kreissektor augeMitte;
    private Kreissektor augeLinksOben;
    private Kreissektor augeLinksMitte;
    private Kreissektor augeLinksUnten;
    private Kreissektor augeRechtsOben;
    private Kreissektor augeRechtsMitte;
    private Kreissektor augeRechtsUnten;
    private Rechteck markierung;
    private Beschriftung kuerzel;

    /**
     * Konstruktor.
     * Erzeuge einen Wrfel, dessen linke obere Ecke bei (x|y)
     * liegt, dessen Flche die Farbe "flaechenfarbe" hat und
     * dessen Augen die Farbe "augenfarbe" haben.
     */
    public Wuerfel (int x, int y, String flaechenfarbe, String augenfarbe, char buchstabe)
    {
        haeufigkeit = new int[7];

        wuerfelflaeche = new Rechteck ();
        wuerfelflaeche.setzeBreite (100);
        wuerfelflaeche.setzeHoehe (100);
        wuerfelflaeche.setzeFarbe (flaechenfarbe);
        wuerfelflaeche.setzePosition (x, y);
        wuerfelflaeche.macheSichtbar ();

        markierung = new Rechteck ();
        markierung.setzeBreite (100);
        markierung.setzeHoehe (5);
        markierung.setzeFarbe ("rot");
        markierung.setzePosition (x, y + 105);
        markierung.macheSichtbar ();

        markiert = true;

        augeLinksOben = new Kreissektor ();
        augeLinksOben.setzeGesamtwinkel (360);
        augeLinksOben.setzeRadius (10);
        augeLinksOben.setzePosition (x + 10, y + 10);
        augeLinksOben.setzeFarbe (augenfarbe);
        augeLinksOben.macheSichtbar ();

        augeLinksMitte = new Kreissektor ();
        augeLinksMitte.setzeGesamtwinkel (360);
        augeLinksMitte.setzeRadius (10);
        augeLinksMitte.setzePosition (x + 10, y + 40);
        augeLinksMitte.setzeFarbe (augenfarbe);
        augeLinksMitte.macheSichtbar ();

        augeLinksUnten = new Kreissektor ();
        augeLinksUnten.setzeGesamtwinkel (360);
        augeLinksUnten.setzeRadius (10);
        augeLinksUnten.setzePosition (x + 10, y + 70);
        augeLinksUnten.setzeFarbe (augenfarbe);
        augeLinksUnten.macheSichtbar ();

        augeMitte = new Kreissektor ();
        augeMitte.setzeGesamtwinkel (360);
        augeMitte.setzeRadius (10);
        augeMitte.setzePosition (x + 40, y + 40);
        augeMitte.setzeFarbe (augenfarbe);
        augeMitte.macheSichtbar ();

        augeRechtsOben = new Kreissektor ();
        augeRechtsOben.setzeGesamtwinkel (360);
        augeRechtsOben.setzeRadius (10);
        augeRechtsOben.setzePosition (x + 70, y + 10);
        augeRechtsOben.setzeFarbe (augenfarbe);
        augeRechtsOben.macheSichtbar ();

        augeRechtsMitte = new Kreissektor ();
        augeRechtsMitte.setzeGesamtwinkel (360);
        augeRechtsMitte.setzeRadius (10);
        augeRechtsMitte.setzePosition (x + 70, y + 40);
        augeRechtsMitte.setzeFarbe (augenfarbe);
        augeRechtsMitte.macheSichtbar ();

        augeRechtsUnten = new Kreissektor ();
        augeRechtsUnten.setzeGesamtwinkel (360);
        augeRechtsUnten.setzeRadius (10);
        augeRechtsUnten.setzePosition (x + 70, y + 70);
        augeRechtsUnten.setzeFarbe (augenfarbe);
        augeRechtsUnten.macheSichtbar ();
        
        kuerzel = new Beschriftung ();
        kuerzel.setzeInhalt (new String ().valueOf (buchstabe));
        kuerzel.setzeSchriftgroesse (18);
        kuerzel.setzeFarbe ("schwarz");
        kuerzel.setzePosition (x + 45, y + 125);
        kuerzel.macheSichtbar ();

        augenzahl = 6;

        zeichne ();
    }

    /**
     * Zeichne den Wrfel passend zur Augenzahl.
     */
    private void zeichne ()
    {
        switch (augenzahl)
        {
            case 1:
            augeLinksOben.macheUnsichtbar ();
            augeLinksMitte.macheUnsichtbar ();
            augeLinksUnten.macheUnsichtbar ();
            augeRechtsOben.macheUnsichtbar ();
            augeRechtsMitte.macheUnsichtbar ();
            augeRechtsUnten.macheUnsichtbar ();
            augeMitte.macheSichtbar ();
            break;

            case 2:
            augeLinksOben.macheUnsichtbar ();
            augeLinksMitte.macheUnsichtbar ();
            augeLinksUnten.macheSichtbar ();
            augeRechtsOben.macheSichtbar ();
            augeRechtsMitte.macheUnsichtbar ();
            augeRechtsUnten.macheUnsichtbar ();
            augeMitte.macheUnsichtbar ();
            break;

            case 3:
            augeLinksOben.macheUnsichtbar ();
            augeLinksMitte.macheUnsichtbar ();
            augeLinksUnten.macheSichtbar ();
            augeRechtsOben.macheSichtbar ();
            augeRechtsMitte.macheUnsichtbar ();
            augeRechtsUnten.macheUnsichtbar ();
            augeMitte.macheSichtbar ();
            break;

            case 4:
            augeLinksOben.macheSichtbar ();
            augeLinksMitte.macheUnsichtbar ();
            augeLinksUnten.macheSichtbar ();
            augeRechtsOben.macheSichtbar ();
            augeRechtsMitte.macheUnsichtbar ();
            augeRechtsUnten.macheSichtbar ();
            augeMitte.macheUnsichtbar ();
            break;

            case 5:
            augeLinksOben.macheSichtbar ();
            augeLinksMitte.macheUnsichtbar ();
            augeLinksUnten.macheSichtbar ();
            augeRechtsOben.macheSichtbar ();
            augeRechtsMitte.macheUnsichtbar ();
            augeRechtsUnten.macheSichtbar ();
            augeMitte.macheSichtbar ();
            break;

            case 6:
            augeLinksOben.macheSichtbar ();
            augeLinksMitte.macheSichtbar ();
            augeLinksUnten.macheSichtbar ();
            augeRechtsOben.macheSichtbar ();
            augeRechtsMitte.macheSichtbar ();
            augeRechtsUnten.macheSichtbar ();
            augeMitte.macheUnsichtbar ();
            break;
        }
    }

    /**
     * Wrfle mit dem Wrfel.
     */
    public void wuerfle ()
    {
        if (markiert)
        {
            augenzahl = (int) Math.floor (6 * Math.random () + 1);
            haeufigkeit[augenzahl] = haeufigkeit[augenzahl] + 1;

            zeichne ();
        }
    }

    /**
     * Markiere den Wrfel.
     */
    public void markiere ()
    {
        markiert = true;
        markierung.macheSichtbar ();
    }
    
    /**
     * ndere die Markierung.
     */
    public void aendereMarkierung ()
    {
        if (markiert)
        {
            markiert = false;
            markierung.macheUnsichtbar ();
        }
        else
        {
            markiert = true;
            markierung.macheSichtbar ();
        }
    }

    /**
     * Gib die Hufigkeitsverteilung auf die Konsole aus.
     */
    public void druckeStatistik ()
    {
        int i;

        for (i = 1; i <= 6; i = i + 1)
        {
            System.out.println ("Augenzahl " + i + ": " + haeufigkeit[i] + "-mal");
        }
    }

    /**
     * Nenne die Augenzahl.
     */
    public int gibAugenzahl ()
    {
        return augenzahl;
    }
}
